%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [dxnulloverX,varargout] = DisparityAveraging_RFconv(dt,T,weightmethod,varargin)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Implementation of our disparity averaging model for depth in the
% stroboscopic Pulfrich stimulus.
% In this version, the weight function is the convolution of RF functions.
% The RF function can be:
% Gaussian, in which case the 3rd argument must specify tau for the Gaussian: exp(-t^2/2/tau^2)
% half-Gaussian, in which case the 3rd argument must specify tau for the Gaussian: (t>0).*exp(-t^2/2/SD^2)
% exponential, in which case the 3rd argument must specify tau for the exponential: (t>0).*exp(-t/tau)
% It outputs dxoverX.
dtoverT = dt/T;
for k = 1:length(dtoverT)
    j=0;
    w(k) = weight(dt(k),weightmethod,varargin{:});
    denom = w(k);
    numer = 0;
    
    dxnulloverX(k) = Inf;
    fracchange = Inf;
    while fracchange>1e-6
        j=j+1;
		wplus = weight(j*T+dt(k),weightmethod,varargin{:});
		wminus = weight(-j*T+dt(k),weightmethod,varargin{:});
        numer = numer + j*wplus - j*wminus;
        denom = denom + wplus + wminus;
        dxnulloverXprev = dxnulloverX(k);
        dxnulloverX(k) = -numer/denom;
        fracchange = abs((dxnulloverXprev-dxnulloverX(k))/dxnulloverX(k));
    end
end
if nargout>1
    varargout{1} = w;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function w = weight(delay,weightmethod,varargin)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
switch weightmethod
case 'Gaussian'
    % RF is a Gaussian with SD tau; fall-off is autocorrelation of this RF
    tau = varargin{1};
    w = exp(-delay.^2./4./tau.^2);
case 'halfGaussian'
    % RF is a half-Gaussian with SD tau; fall-off is autocorrelation of this RF
    tau = varargin{1};
    w = exp(-delay.^2./4./tau.^2).*(1-erf(abs(delay)/tau/2));
case 'exponential'
    % RF is a exponential with timeconstant tau; fall-off is autocorrelation of this RF
    tau = varargin{1};
    w = exp(-abs(delay)./tau);
case 'test1'
    tau = varargin{1};
    TF = varargin{2};
    t = [-10*tau:0.1*tau: 10*tau + delay];
    r = (t<=0).*exp(-t.^2./2/tau^2) .* cos(2*pi*TF.*t); % test1
    rshift = ((t-delay)<=0).*exp(-(t-delay).^2./2/tau^2) .* cos(2*pi*TF.*(t-delay));
    w = sum(r.*rshift);
case 'Gabor'
	% Temporal RF is (t<=0).*exp( - (t+temporalag).^2./twoSDtsq ) .* cos(twopiTF.*(t+temporalag))
    tau = varargin{1};
    TF = varargin{2};
	w = exp(-delay.^2./4/tau.^2) .* ( cos(2*pi*TF.*delay) + exp(-4*pi^2*tau^2.*TF^2) );
end
