% This version is for a Gaussian RF (Gaussian profiles in both space and time), tuned to the apparent velocity of the stimulus
function SimulationForFigure11 % aka StrobePulfrich_disponly_WTA
clear all
close all hidden
format compact
format long
global twoSDxsq twoSDtsq costheta sintheta SDt temporalag theta alphasq
global ntime ndisparity tol
% This is based on the maths I have laid out in longversion.doc.

% This program outputs the response of a population of model neurons tuned
% to different stimulus positions and disparities, at different times
% throughout a strobe Pulfrich stimulus

% Define stimulus properties
TToverSDt = [6:-1:2]; % temporal period of strobe stimulus, in s
vel = 3.6; % speed of strobe stimulus, in deg/s
dttoverT = [0 0.1 0.2:0.05:0.5];

% Define RF function
SDt = 10e-3; %s
temporalag = 50e-3;
theta = atan(vel); % this is what makes the RF tilted in spacetime
alphasq = 10;
costheta=cos(theta);
sintheta=sin(theta);
SDx = SDt*sqrt(sintheta^2/(costheta^2+sintheta^2/alphasq) + costheta^2/(alphasq*costheta^2+sintheta^2));

% PArameters that control the accuracy. 
ndisparity = 101;
nxcen = 41;
ntime = 121;
tol = 5e-6;
% Files ending _accA.mat have ndisparity=101, nxcen=41, ntime=121, tol=5e-6;
% Files ending _accB.mat have ndisparity=151, nxcen=101, ntime=161, tol=2e-6; (ie more accurate)

twoSDxsq = 2*SDx^2;
twoSDtsq = 2*SDt^2;

% NB to make it non-tilted: theta=0;costheta=1;sintheta=0;SDx=0.1;alphasq=(SDt/SDx)^2;

t=[-(temporalag+3*SDt):0.001:0];
x=[-3*SDx:0.001:3*SDx];
[t2,x2]=meshgrid(t,x);
imagesc(t,x,RF(x2,t2))
xlabel('time (s)')
ylabel('position (deg)')
title('Receptive field')
drawnow
hold on
plot(t,vel*(t+temporalag),'y')
axis equal
pause

col={'r' 'b' [0 0.5 0] 'm' 'g' 'c'};
figure
for jT=1:length(TToverSDt)
    T=TToverSDt(jT)*SDt;
    X = vel .* T; % spatial period of strobe stimulus, in deg
    time = [0:ntime-1]*T/ntime;
    for jdt=1:length(dttoverT)
        dtoverT=dttoverT(jdt);
        Dt = dtoverT*T;
        % Calculate the maximum disparity we can encounter
        dispmax = 4 * SDx + abs(vel) * (4*SDt + abs(Dt));
        dispmin = -dispmax;
        disparity = [ dispmin : (dispmax-dispmin)/(ndisparity-1) : dispmax  ] ; 
        
        vL = zeros(ntime,nxcen,ndisparity);
        vR = vL;
        for jt=1:ntime    
            xmin = - 4 * SDx - abs(X);
            xmax = 4 * SDx + abs(X);
            xcen = [ xmin : (xmax-xmin)/(nxcen-1) : xmax]  + vel*time(jt) - vel*temporalag;
            for jx=1:nxcen
                for jd=1:ndisparity
                    [vL(jt,jx,jd),vR(jt,jx,jd)] = Convolutions(time(jt),X,T,Dt,xcen(jx),disparity(jd));
                end
            end
            imagesc(disparity,xcen,squeeze(vL(jt,:,:).*vR(jt,:,:)))
            drawnow
        end
		try
			save(sprintf('Fullresults_simple_gauss_theta=%ddeg_SDx=%4.3f_Tontau=%1d_jdt%d_accA.mat',...
                round(theta*180/pi),SDx,TToverSDt(jT),jdt),...
                'SDx','SDt','theta','temporalag','vel','dttoverT','ndisparity','nxcen','ntime','tol','vL','T','X','vR');
		catch
			save(sprintf('C:\Fullresults_simple_gauss_theta=%ddeg_SDx=%4.3f_Tontau=%1d_jdt%d_accA.mat',...
                round(theta*180/pi),SDx,TToverSDt(jT),jdt),...
                'SDx','SDt','theta','temporalag','vel','dttoverT','ndisparity','nxcen','ntime','tol','vL','T','X','vR');
		end

    end % next delay dt
end % next strobe period T


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [vL,vR] = Convolutions(t,X,T,Dt,xcen,disparity)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
global tol
abstol=1e-30;

jmaxL = floor( t/T);
kmaxR = floor( (t-Dt)/T);

j = jmaxL;
jsm = 0;
fracerrj = 1;
while abs(jsm)>abstol & fracerrj > tol | j>jmaxL-15 
    % To avoid making it spend ages converging accurately on a tiny value,
    % I force it to loop at least 15 times, but then stop if the answer is
    % still no larger than abstol.
    % NB! This can screw things up if the temporal kernel is such that uou
    % need to you back more than 15 periods to get a response
    contrib = RF(j*X - xcen - disparity./2 , j*T-t)  ;
    jsm = jsm + contrib;
    j = j - 1;
    if abs(jsm)>0
        fracerrj = abs(contrib/jsm);
    end
end     
vL = jsm;

k = kmaxR;
ksm = 0;
fracerrk = 1;
while abs(ksm)>abstol & fracerrk > tol | k>kmaxR-15 
    contrib = RF(k*X - xcen + disparity./2 , k*T+Dt-t )  ;
    ksm = ksm + contrib;
    k = k - 1;
    if abs(ksm)>0
        fracerrk = abs(contrib/ksm);
    end
end     
vR = ksm;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function r = RF(x,t)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
global twoSDxsq twoSDtsq temporalag costheta sintheta alphasq
% Define RF function:
tiltt = x.*sintheta + (t+temporalag)*costheta;
tiltx = x.*costheta - (t+temporalag)*sintheta;
twoSD1sq = twoSDtsq/(costheta^2+1/alphasq*sintheta^2);
twoSD2sq = twoSDtsq/(alphasq*costheta^2+sintheta^2);
r = (t<=0).*exp( - tiltt.^2./twoSD1sq - tiltx.^2./twoSD2sq) ;

